use rand::rngs::OsRng;
use x25519_dalek::{EphemeralSecret, PublicKey, StaticSecret};
use crate::noise::handshake::HandshakeState;
use crate::noise::handshake::initiator::{handshake_init_from, handshake_init_to};
use crate::noise::handshake::response::{handshake_response_from, handshake_response_to};
use crate::qcrypto::pki::qcrypto_dh_generate_longterm;

#[test]
fn noise_halfhandshake_test() {
    let alice_keypair = qcrypto_dh_generate_longterm();
    let bob_keypair = qcrypto_dh_generate_longterm();

    let mut alice_session = HandshakeState {
        h: [0u8; 32],
        ck: [0u8; 32],
        e_pub_i: PublicKey::from([0u8; 32]),
        e_pub_r: PublicKey::from([0u8; 32]),
        s_pub_i: PublicKey::from([0u8; 32]),
        s_pub_r: PublicKey::from([0u8; 32]),
        e_priv_me: StaticSecret::new(OsRng),
        s_priv_me: alice_keypair.0,
        s_pub_them: bob_keypair.1,
        i_i: 0,
        i_r: 0,
        q: [0u8; 32],
        cookies: vec![],
    };
    let mut bob_session = HandshakeState {
        h: [0u8; 32],
        ck: [0u8; 32],
        e_pub_i: PublicKey::from([0u8; 32]),
        e_pub_r: PublicKey::from([0u8; 32]),
        s_pub_i: PublicKey::from([0u8; 32]),
        s_pub_r: PublicKey::from([0u8; 32]),
        e_priv_me: StaticSecret::new(OsRng),
        s_priv_me: bob_keypair.0,
        s_pub_them: alice_keypair.1,
        i_i: 0,
        i_r: 0,
        q: [0u8; 32],
        cookies: vec![],
    };

    let handshake_init = handshake_init_to(&mut alice_session).unwrap();
    handshake_init_from(&mut bob_session, handshake_init).unwrap();

    println!("{:?}", alice_session);
    println!("{:?}", bob_session);

    assert!(alice_session.is_eq(&bob_session));
}

#[test]
fn noise_nocookie_handshake_test() {
    let alice_keypair = qcrypto_dh_generate_longterm();
    let bob_keypair = qcrypto_dh_generate_longterm();

    let mut alice_session = HandshakeState {
        h: [0u8; 32],
        ck: [0u8; 32],
        e_pub_i: PublicKey::from([0u8; 32]),
        e_pub_r: PublicKey::from([0u8; 32]),
        s_pub_i: PublicKey::from([0u8; 32]),
        s_pub_r: PublicKey::from([0u8; 32]),
        e_priv_me: StaticSecret::new(OsRng),
        s_priv_me: alice_keypair.0,
        s_pub_them: bob_keypair.1,
        i_i: 0,
        i_r: 0,
        q: [0u8; 32],
        cookies: vec![],
    };
    let mut bob_session = HandshakeState {
        h: [0u8; 32],
        ck: [0u8; 32],
        e_pub_i: PublicKey::from([0u8; 32]),
        e_pub_r: PublicKey::from([0u8; 32]),
        s_pub_i: PublicKey::from([0u8; 32]),
        s_pub_r: PublicKey::from([0u8; 32]),
        e_priv_me: StaticSecret::new(OsRng),
        s_priv_me: bob_keypair.0,
        s_pub_them: alice_keypair.1,
        i_i: 0,
        i_r: 0,
        q: [0u8; 32],
        cookies: vec![],
    };

    let handshake_init = handshake_init_to(&mut alice_session).unwrap();
    handshake_init_from(&mut bob_session, handshake_init).unwrap();

    let handshake_response = handshake_response_to(&mut bob_session).unwrap();
    handshake_response_from(&mut alice_session, handshake_response).unwrap();

    println!("{:?}", alice_session);
    println!("{:?}", bob_session);

    assert!(alice_session.is_eq(&bob_session));
}