//! An implementation of the HKDF algorithm

use crate::qcrypto::hashes::{qcrypto_hmac, qcrypto_hmac_twice};

#[allow(clippy::module_name_repetitions)]
/// Given a constant order N, a varied length key and varied length input, perform the HKDF algorithm and produce an N-array of 32-byte arrays.
pub fn qcrypto_hkdf<const N: usize>(key: &[u8], input: &[u8]) -> [[u8; 32]; N] {
    let mut result_array = [[0u8; 32]; N];

    let t0 = qcrypto_hmac(key, input);
    let t1 = qcrypto_hmac(&t0, &[1u8]);

    result_array[0] = t1;

    for n in 1..N {
        result_array[n] = qcrypto_hmac_twice(&t0, &result_array[n-1], &n.to_be_bytes());
    }

    result_array
}