//! A module to implement all of the various cryptography constructs required to implement `WireGuard`

use tai64::Tai64N;

pub mod hashes;
pub mod pki;
pub mod aead;

#[cfg(test)]
pub mod tests;
pub mod hkdf;

/// Get the current TAI64N timestamp
pub fn timestamp() -> Tai64N {
    Tai64N::now()
}

/// The cryptography handshake construction identifier
pub const CONSTURCTION: &str = "Noise_IKpsk2_25519_ChaChaPoly_BLAKE2s";
/// The WireGuard protocol identifier
pub const IDENTIFIER: &str = "WireGuard v1 zx2c4 Jason@zx2c4.com";
/// The MAC1 cookie label
pub const LABEL_MAC1: &str = "mac1----";
/// The COOKIE1 cookie label
pub const LABEL_COOKIE: &str = "cookie--";