//! Various public-key cryptography functions

use rand::rngs::OsRng;
use x25519_dalek::{EphemeralSecret, PublicKey, SharedSecret, StaticSecret};

type LongtermKeypair = (StaticSecret, PublicKey);

/// Generate a X25519 keypair
pub fn qcrypto_dh_generate_longterm() -> LongtermKeypair {
    let secret = StaticSecret::new(OsRng);
    let public = PublicKey::from(&secret);
    (secret, public)
}

/// Perform Elliptic-Curve Diffie-Hellman between a secret and public key to get a shared secret value
pub fn qcrypto_dh_longterm(secret: &StaticSecret, public: &PublicKey) -> SharedSecret {
    secret.diffie_hellman(public)
}


type EphemeralKeypair = (EphemeralSecret, PublicKey);

/// Generate a X25519 keypair
pub fn qcrypto_dh_generate_ephemeral() -> EphemeralKeypair {
    let secret = EphemeralSecret::new(OsRng);
    let public = PublicKey::from(&secret);
    (secret, public)
}

/// Perform Elliptic-Curve Diffie-Hellman between a secret and public key to get a shared secret value
pub fn qcrypto_dh_ephemeral(secret: EphemeralSecret, public: &PublicKey) -> SharedSecret {
    secret.diffie_hellman(public)
}